﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ContactList
{
    public partial class Form1 : Form
    {
        private List<Contact> contacts = new List<Contact>();

        public Form1()
        {
            InitializeComponent();
        }

        private void NewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            contacts = new List<Contact>();
            UpdateGUI();
        }

        private void UpdateGUI()
        {
            listBox1.Items.Clear();
            foreach(Contact contact in contacts)
            {
                listBox1.Items.Add(contact);
            }
            contactsCountStatusLabel.Text = contacts.Count.ToString();
        }

        private void OpenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if(openFileDialog1.ShowDialog()== DialogResult.OK)
            {
                try
                {
                    LoadContacts(openFileDialog1.FileName);
                }
                catch(InvalidDataException ex)
                {
                    MessageBox.Show(String.Format("Chyba vstupnich dat:{0}", ex.Message), "Chyba nacitani");
                }
            }
        }

        private void LoadContacts(string filename)
        {
            using (StreamReader reader = new StreamReader(filename))
            {
                contacts.Clear();
                string line;
                while((line=reader.ReadLine())!= null)
                {
                    Contact contact = LineToContact(line);
                    contacts.Add(contact);
                }
            }
            UpdateGUI();
        }

        private Contact LineToContact(string line)
        {
            Contact contact;
            string[] contactParts = line.Split(':');
            if (contactParts.Length != 4)
            {
                throw new InvalidDataException("Invalid contact format.");
            }
            contact = ContactPartsToContact(contactParts);
            return contact;
        }

        private Contact ContactPartsToContact(string[] contactParts)
        {
            string[] addressParts = contactParts[3].Split(';');
            if (addressParts.Length != 4)
            {
                throw new InvalidDataException("Invalid contact format.");
            }    
            Contact contact = new Contact();
            contact.FirstName = contactParts[0];
            contact.Surname = contactParts[1];
            contact.Phone = contactParts[2];
            contact.Address = addressPartsToAddress(addressParts);
            return contact;
        }

        private Address addressPartsToAddress(string[] addressParts)
        {
            Address addres = new Address();
            addres.Street = addressParts[0];
            try
            {
                addres.HouseNumber = uint.Parse(addressParts[1]);
            }
            catch (Exception ex)
            {
                throw new InvalidDataException("Invalid address.", ex);
            }
            addres.PostCode = addressParts[2];
            addres.Country = addressParts[3];
            return addres;
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if( saveFileDialog1.ShowDialog() != DialogResult.OK )
            {
                SaveContacts(saveFileDialog1.FileName);
            }
        }

        private void SaveContacts(string fileName)
        {
            using (StreamWriter writer = new StreamWriter(fileName))
            {
                foreach (Contact contact in contacts)
                {
                    writer.WriteLine(contact);
                }
            }
        }

        private void konecToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void addToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AddContactDialog dialog = new AddContactDialog();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                contacts.Add(dialog.contact);
                UpdateGUI();
            }
        }

        private void removeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (listBox1.SelectedIndex >= 0)
            {
                contacts.RemoveAt(listBox1.SelectedIndex);
                UpdateGUI();
            }
            else
            {
                MessageBox.Show("Nutno vybrat odebirany prvek","Chyba!");
            }
        }
    }
}
